# -*- coding:utf-8 -*-
'''
Created on 2020年6月20日

@author: siwenwei
'''

import math
from ..html import elements


class PieChart(elements.Svg):
    """饼图"""

    DATA_KEY = "data"
    LABEL_KEY = "label"
    COLOR_KEY = "color"
    PIE_CHART_CSS_CLASS_KEY = "pie_chart_css_class"  # 饼图样式
    LEGEND_CSS_CLASS_KEY = "legend_css_class"  # 图例样式
    LEGEND_TEXT_CSS_CLASS_KEY = "legend_text_css_class"  # 图例文本样式

    def __init__(self, parts, width, height, cx, cy, r):
        """创建一个<svg>元素，并在其中绘制一个饼状图

        Args:
            parts: 用于绘制的扇形数据的数组，数组每一项都表示饼状图的一个楔，[part,...], part={"data":12,"label":"成功","color":"antiquewhite"}
            width,height: SVG图形的大小，单位为像素
            cx cy r :  饼状图的圆心及半径

        Returns:
            一个保存饼状图的<svg>元素
        """
        super().__init__(width, height)
        self.r = r
        self.cx = cx
        self.cy = cy
        self.parts = parts
        self.width = width
        self.height = height

        self.lx = self.cx + self.r * 1.5
        self.ly = (self.cy + self.r * 2) / 6

        self._build()

    @classmethod
    def build_part(cls, data, label, color, **css_classes):
        """构建饼图数据

        Args:
            data: 用于绘制的数字,饼图大小
            label: 饼图名称
            color：饼图颜色
        """

        part = {cls.DATA_KEY: data, cls.LABEL_KEY: label, cls.COLOR_KEY: color}
        EXCLUDE_KEYS = [cls.DATA_KEY, cls.LABEL_KEY, cls.COLOR_KEY]
        for k, v in css_classes.items():
            if k not in EXCLUDE_KEYS:
                part[k] = v

        return part

    @classmethod
    def percent(cls, divisor, dividend, precision=2):
        fmt = "{:." + str(precision) + "f}%"
        return fmt.format(divisor / dividend * 100)

    @classmethod
    def attach_suffix(cls, text, suffix):
        fmt = "{}({})"
        return fmt.format(text, suffix)

    def _build(self):

        viewbox = "{x} {y} {width} {height}".format(x="0", y="0", width=self.width, height=self.height)
        self.set_attr("viewBox", viewbox)

        # 累加datas的值，以便于知道饼状图的大小
        total = 0
        for part in self.parts:
            total = total + part[self.DATA_KEY]

        # 现在计算出饼状图每个分片的大小，其中角度以弧度制计算
        angle_parts = []
        for part in self.parts:
            data = part[self.DATA_KEY]
            angle = data / total * 360
            angle = 359.9 if angle == 360 else angle
            radian = angle / 180 * math.pi
            angle_parts.append((radian, part))

        # 遍历饼状图的每个分片
        startangle = 0
        for i, angle_part in enumerate(angle_parts):

            angle, part = angle_part
            # 这里表示楔的结束位置
            endagle = startangle + angle

            # 计算出楔和园相交的两个点
            # 这些计算公式都是以12点钟方向为0度
            # 顺时针方向角度递增
            x1 = int(self.cx) + int(self.r) * math.sin(startangle)
            y1 = int(self.cy) - int(self.r) * math.cos(startangle)
            x2 = int(self.cx) + int(self.r) * math.sin(endagle)
            y2 = int(self.cx) - int(self.r) * math.cos(endagle)

            # 这个标记表示角度大于半圆
            # 此标记在绘制svg弧形组件的时候需要
            big = 0
            if endagle - startangle > math.pi:
                big = 1
            data = part[self.DATA_KEY]
            label = part[self.LABEL_KEY]
            color = part[self.COLOR_KEY]

            # 使用<svg:path>元素来描述楔
            svg_path = self.draw_svg_path(label)

            # 下面的字符串包含路径的详细信息
            # 设置<svg:path>元素的属性
            svg_path.M(self.cx, self.cy).L(x1, y1).A(x2, y2, self.r, self.r, big, "0", "1").Z().set_d_attr()
            svg_path.set_attr("fill", color)  # 设置楔的颜色
            svg_path.set_attr("stroke", "black")  # 楔的外边框为黑色
            svg_path.set_attr("stroke-width", "0.7")  # 两个单位宽
            if self.PIE_CHART_CSS_CLASS_KEY in part:
                svg_path.add_css_class(part[self.PIE_CHART_CSS_CLASS_KEY])

            # 当前楔的结束就是下一个楔的开始
            startangle = endagle

            # 现在绘制一些相应的小方块来表示图例
            icon_width = 70
            icon = self.draw_svg_rect(label)
            icon.set_attr("x", self.lx)  # 定位小方块
            icon.set_attr("y", str(int(self.ly) + 30 * i))
            icon.set_attr("width", icon_width)  # 设置小方块的大小
            icon.set_attr("height", "7")
            icon.set_attr("fill", color)  # 填充小方块的颜色和对应的楔的颜色相同
            icon.set_attr("stroke", "black")  # 子外边框颜色也相同
            icon.set_attr("stroke-width", "0.7")
            if self.LEGEND_CSS_CLASS_KEY in part:
                icon.add_css_class(part[self.LEGEND_CSS_CLASS_KEY])

            # 在小方块的右边添加标签
            icon_text = self.draw_svg_text(label, self.attach_suffix(label, self.percent(data, total)))
            icon_text.set_attr("x", str(int(self.lx) + icon_width + 10))  # 定位标签文本
            icon_text.set_attr("y", str(int(self.ly) + 30 * i + 9))  # 文本样式属性还可以通过CSS来设置
            icon_text.set_attr("fill", color)
            icon_text.set_attr("font-size", "16")
            icon_text.set_attr("font-family", "sans-serif")
            if self.LEGEND_TEXT_CSS_CLASS_KEY in part:
                icon_text.add_css_class(part[self.LEGEND_TEXT_CSS_CLASS_KEY])


if __name__ == "__main__":

    html = """<!DOCTYPE html><html><head><title>表格属性</title><meta charset="utf-8" /></head><body>{}</body></html>"""
    width = 500
    height = 260
    cx = 100
    cy = 100
    r = 100
    parts = [
        PieChart.build_part(0, "失败", "red", pie_chart_css_class="fail", legend_css_class="fail-legend", legend_text_css_class="fail-legend-text"),
        PieChart.build_part(27, "成功", "green", pie_chart_css_class="success", legend_css_class="success-legend", legend_text_css_class="success-legend-text"),
        PieChart.build_part(0, "阻塞", "sandybrown", pie_chart_css_class="block", legend_css_class="block-legend", legend_text_css_class="block-legend-text"),
        PieChart.build_part(0, "异常", "antiquewhite", pie_chart_css_class="error", legend_css_class="error-legend", legend_text_css_class="error-legend-text")
    ]
    pie = PieChart(parts, width, height, cx, cy, r)
    with open(r"D:\tmsuitest\sp.html", "w", encoding="utf-8") as f:
        f.write(html.format(pie.to_html()))



# -*- coding: utf-8 -*-
'''
Created on 2019年12月5日

@author: siwenwei
'''

import os
import sys
import copy

from .html import elements
from .widgets import tables

if getattr(sys, 'frozen', False):
    # BASE_DIR = os.path.dirname(os.path.abspath(sys.executable))
    BASE_DIR = sys._MEIPASS
else:
    BASE_DIR = os.path.dirname(os.path.abspath(__file__))


class HtmlReportTemplate(object):

    CSS_AND_JS_DIR = "resources"
    CSS_FILE_NAME = "style.css"
    JS_FILE_NAME = "main.js"
    JQUERY_FILE_NAME = "jquery-1.11.0.min.js"

    def __init__(self, testpoints, title="", task_number="", start_time="", finish_time="", executor="", project_name="", task_description=""):

        self.html = elements.HTML()
        self.title = title
        self.summary_table = tables.SummaryTable(title, task_number, start_time, finish_time, executor, project_name, task_description, self.pie_chart_info(testpoints))
        self.report_table = tables.ReportTable()
        self.testpoints = testpoints
        self.layer = elements.Div().add_css_class("layer")
        self.html.body.append_child(self.layer)

    def pie_chart_info(self, testpoints):

        points = copy.deepcopy(testpoints)
        fail_count = 0
        block_count = 0
        error_count = 0
        pass_count = 0
        skip_count = 0
        xfail_count = 0
        xpass_count = 0
        for p in points:
            fail_count = fail_count + p["fail_count"]
            block_count = block_count + p["block_count"]
            error_count = error_count + p["error_count"]
            pass_count = pass_count + p["pass_count"]
            skip_count = skip_count + p["skip_count"]
            xfail_count = xfail_count + p["xfail_count"]
            xpass_count = xpass_count + p["xpass_count"]
        return (pass_count, fail_count, block_count, error_count, skip_count, xfail_count, xpass_count)

    def _write_metas(self):

        m1 = elements.Meta().set_attr("name", "description").set_attr("content", "auto test html report")
        m2 = elements.Meta().set_attr("name", "author").set_attr("content", "siwenwei")
        m3 = elements.Meta().set_attr("http-equiv", "Content-Type").set_attr("content", "text/html; charset=utf-8")

        self.html.head.append_child(m1).append_child(m2).append_child(m3)

    def _write_inline_style(self):

        style = elements.Style().set_attr("type", "text/css").set_attr("media", "screen")
        with open(os.path.join(BASE_DIR, self.CSS_AND_JS_DIR, self.CSS_FILE_NAME), "r", encoding="utf-8") as f:
            style.text = """\n%s\n""" % f.read()
        self.html.head.append_child(style)

    def _write_jquery_script(self):

        script = elements.Script().set_attr("type", "text/javascript").set_attr("language", "javascript")
        with open(os.path.join(BASE_DIR, self.CSS_AND_JS_DIR, self.JQUERY_FILE_NAME), "r", encoding="utf-8") as f:
            script.text = """<!-- \n%s\n -->""" % f.read()
        self.html.head.append_child(script)

    def _write_inline_script(self):

        script = elements.Script().set_attr("type", "text/javascript").set_attr("language", "javascript")
        with open(os.path.join(BASE_DIR, self.CSS_AND_JS_DIR, self.JS_FILE_NAME), "r", encoding="utf-8") as f:
            script.text = """<!-- \n%s\n -->""" % f.read()
        self.html.head.append_child(script)

    def set_report_title(self):

        self.html.head.title.text = self.title

    def _fill_summary_table(self):

        self.layer.append_child(self.summary_table)

    def _fill_report_table(self):

        testpoints = []
        for testpoint in self.testpoints:
            testpoints.append(testpoint)
        self.report_table.set_testpoints(*tuple(testpoints))
        self.layer.append_child(self.report_table)

    def to_html(self):

        self._write_metas()
        self.set_report_title()
        self._write_inline_style()
        self._write_jquery_script()
        self._write_inline_script()
        self._fill_summary_table()
        self._fill_report_table()
        return self.html.to_html()

    def save_as_file(self, full_file_path):

        with open(full_file_path, "w", encoding="utf-8") as f:
            f.write(self.to_html())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/17
'''
from unittest.runner import TextTestRunner
from .seven_result import SevenTestResult


class SevenTestRunner(TextTestRunner):

    resultclass = SevenTestResult



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/04/22 18:05:27
'''


class AttributeMarker(object):
    def __init__(self, value, final=False, description="", *expend_args, **expend_kwargs):

        self.value = value
        self.final = final
        self.description = description
        self.expend_args = expend_args
        self.expend_kwargs = expend_kwargs

    def __str__(self):

        return "{value} ------> {description}".format(value=self.value, description=self.description)


class ConstAttributeMarker(AttributeMarker):
    def __init__(self, value, description="", *expend_args, **expend_kwargs):

        AttributeMarker.__init__(self, value, True, description, *expend_args, **expend_kwargs)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''

import os
from . import utils
from .attrs_marker import AttributeMarker
from .abstract_data_provider import AbsractDataProvider
from .excel_file_reader import TestCaseExcelFileReader as ExcelReader


class SevenDataProvider(AbsractDataProvider):

    FILE_EXT = AttributeMarker(".xlsx", True, "数据文件拓展名")
    BLOCK_FLAG = AttributeMarker("用例名称", True, "用例分隔标记")
    DEFAULT_SHEET_INDEX = AttributeMarker(0, True, "默认从索引为0的工作表读取数据")

    # get_datasets方法变长字典参数kwargs接收的参数的键名
    PARAM_DATA_FILE_NAME = AttributeMarker("data_file_name", True, "数据文件名称参数")
    PARAM_DATA_FILE_DIR_PATH = AttributeMarker("data_file_dir_path", True, "数据文件所在目录路径参数")
    PARAM_SHEET_NAME_OR_INDEX = AttributeMarker("sheet_name_or_index", True, "数据文件中数据所在的工作表索引(从0开始)或名称参数")
    KWARGS_NAMES = AttributeMarker((PARAM_DATA_FILE_NAME, PARAM_DATA_FILE_DIR_PATH, PARAM_SHEET_NAME_OR_INDEX), True, "接收的参数名")

    def _get_data_file_name(self, kwargs, default_value=None):

        param = self.PARAM_DATA_FILE_NAME
        filename = kwargs.get(param, default_value)
        if utils.is_blank_space(filename):
            raise ValueError("数据文件名必须是字符串类型且不能为空")
        return filename

    def _get_data_file_dir_paht(self, kwargs):

        param = self.PARAM_DATA_FILE_DIR_PATH
        if param not in kwargs.keys():
            raise AttributeError("没有传入数据文件目录")
        dirpath = kwargs[param]
        if utils.is_blank_space(dirpath):
            raise ValueError("数据文件目录必须是字符串类型且不能为空")
        return dirpath

    def _get_sheet_name_or_index(self, kwargs):
        return kwargs.get(self.PARAM_SHEET_NAME_OR_INDEX, self.DEFAULT_SHEET_INDEX)

    def _build_file_full_path(self, data_file_dir_path, data_file_name):
        """构建完整的excel数据文件路径

        Args:
            data_file_dir_path: 文件目录
            data_file_name: 文件名称
        """

        name = data_file_name
        ext = self.FILE_EXT
        if utils.is_blank_space(data_file_dir_path):
            raise ValueError("传入的数据文件目录路径不能为空：{}".format(data_file_dir_path))
        dir_path = data_file_dir_path
        if name and not utils.is_blank_space(name):
            full_name = name if name.endswith(ext) else name + ext
        else:
            raise ValueError("无效数据文件名称：{}".format(name))
        return os.path.join(dir_path, full_name)

    def get_testdatas(self, test_class_name, test_method_name, *args, **kwargs):
        """根据文件名从指定的excel文件(xlsx文件格式)读取出数据, 返回一维列表，每个元素是excel表中一行测试数据信息字典.
        eg: [{"减数1": "36", "减数2": "10", "预期": "26"}, {"减数1": "57", "减数2": "30", "预期": "27"}]

        Args:
            kwargs:
                file_name 数据文件名, 不提供则测试类名称作为文件名
                file_dir_path 数据文件所在目录路径
                sheet_index_or_name Excel工作表索引(从0开始)或名称,不提供则默认取索引0的工作表
        """

        datasets = []

        filename = self._get_data_file_name(kwargs, test_class_name)
        dirpath = self._get_data_file_dir_paht(kwargs)
        full_file_path = self._build_file_full_path(dirpath, filename)

        reader = ExcelReader(full_file_path, testcase_block_separators=self.BLOCK_FLAG, sheet_index_or_name=self._get_sheet_name_or_index(kwargs))
        datas_blocks = reader.load_testcase_data()
        for block in datas_blocks:
            if block.name == test_method_name:
                for row in block.datas:
                    line = {}
                    for cell in row:
                        for title, value in cell.items():
                            if title in line.keys():
                                continue
                            else:
                                line[title] = value
                    datasets.append(line)
                break
        return datasets



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''


class GlobalConfig(object):
    def __init__(self):

        self.__settings = {}

    @property
    def seven_data_provider_data_file_dir(self):

        return self.get('seven_data_provider_data_file_dir', None)

    @seven_data_provider_data_file_dir.setter
    def seven_data_provider_data_file_dir(self, dirpath):

        self.set('seven_data_provider_data_file_dir', dirpath)
        return self

    def set(self, key, val):

        self.__settings[key] = val

    def get(self, key, default_value=None):

        return self.__settings.get(key, default_value)

    def __contains__(self, key):

        return key in self.__settings


GLOBAL_CONFIG = GlobalConfig()
